package org.jetbrains.remoter;

import com.intellij.openapi.components.ApplicationComponent;
import com.intellij.openapi.diagnostic.Logger;
import org.apache.xmlrpc.*;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.ServerSocket;

/**
 * XML-RPC HTTP based server. Based on IDEA 6.0 reversed code but contains
 * some tweaks and using another port.
 * @author Roge Nihsak
 */
public class HttpRpcServerImpl implements ApplicationComponent {

    private static final Logger LOG = Logger.getInstance("#org.jetbrains.remoter.HttpRpcServerImpl");
    public static final int PORT_NUMBER = 16161;
    private CustomWebServer webServer;
    private static final String PORT_PROPERTY_NAME = "http.rpc.port";


    public String getComponentName() {
        return "HtmlRpcServer";
    }

    public void initComponent() {
        if (!available())
            return;
        try {
            webServer = new CustomWebServer(getPortNumber(), null, new XmlRpcServer() {

                protected XmlRpcWorker createWorker() {
                    return new XmlRpcWorker(getHandlerMapping()) {

                        public byte[] execute(InputStream inputstream, String s, String s1) {
                            try {
                                byte abyte0[] = super.execute(inputstream, s, s1);
                                return abyte0;
                            } finally {
                                reset();
                            }
                        }

                        public byte[] execute(InputStream inputstream, XmlRpcContext xmlrpccontext) {
                            try {
                                byte abyte0[] = new byte[0];
                                abyte0 = super.execute(inputstream, xmlrpccontext);
                                return abyte0;
                            } finally {
                                reset();
                            }
                        }

                        private void reset() {
                            requestProcessor = new XmlRpcRequestProcessor();
                        }

                        {
                            responseProcessor = new XmlRpcResponseProcessor() {

                                public byte[] encodeResponse(Object responseParam, String encoding) throws IOException, UnsupportedEncodingException, XmlRpcException {
                                    return responseParam.toString().getBytes();
                                }
                            };

                        }
                    };
                }

            });
            webServer.setParanoid(true);
            webServer.acceptClient("127.0.0.1");
            webServer.start();
        }
        catch (Exception exception) {
            LOG.error(exception);
            webServer = null;
        }
    }

    public int getPortNumber() {
        return getPort();
    }

    private static int getPort() {
        if (System.getProperty(PORT_PROPERTY_NAME) != null)
            return Integer.parseInt(System.getProperty(PORT_PROPERTY_NAME));
        else
            return PORT_NUMBER;
    }

    private static boolean available() {
        ServerSocket serversocket = null;
        try {
            serversocket = new ServerSocket(getPort());
            try {
                serversocket.close();
            }
            catch (IOException ioexception) {
                LOG.error(ioexception);
                return false;
            }
        } catch (Exception e) {
            LOG.error(e);
            boolean flag = false;
            if (serversocket != null)
                try {
                    serversocket.close();
                }
                catch (IOException ioexception1) {
                    LOG.error(ioexception1);
                    return false;
                }
            return flag;
        }
        return true;
    }

    public void disposeComponent() {
        if (webServer != null)
            webServer.shutdown();
    }

    public void addHandler(String s, Object obj) {
        if (webServer != null)
            webServer.addHandler(s, obj);
        else
            LOG.info("Handler not registered because XML-RPC server is not running");
    }

    public void removeHandler(String s) {
        if (webServer != null)
            webServer.removeHandler(s);
    }

}
