/*
* Copyright (c) 2005 JetBrains s.r.o. All Rights Reserved.
*/
package org.jetbrains.remoter;

import com.intellij.ide.plugins.PluginInstaller;
import com.intellij.ide.plugins.PluginNode;
import com.intellij.openapi.application.Application;
import com.intellij.openapi.application.ApplicationManager;
import com.intellij.openapi.application.ModalityState;
import com.intellij.openapi.components.ApplicationComponent;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.extensions.PluginId;
import com.intellij.openapi.progress.ProgressIndicator;
import com.intellij.openapi.progress.ProgressManager;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.project.ProjectManager;
import com.intellij.openapi.ui.Messages;
import com.intellij.openapi.util.text.StringUtil;
import org.jetbrains.annotations.NonNls;

import java.io.*;
import java.net.URLDecoder;
import java.util.HashMap;
import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
 * Installing plugins using IDEA built in mechanism.
 * @author Roge Nihsak
 */
public class PluginLoaderXmlRpcHandler implements ApplicationComponent {

    private static final Logger LOG = Logger.getInstance("#org.jetbrains.remoter.PluginLoaderXmlRpcHandler");
    private static final String HANDLER_NAME = "pluginLoader";
    private final HttpRpcServerImpl myXmlRpcServer;
    private HashMap<String, String> templates = new HashMap<String, String>();


    public PluginLoaderXmlRpcHandler(HttpRpcServerImpl xmlRpcServer) {
        myXmlRpcServer = xmlRpcServer;
    }

    @NonNls
    public String getComponentName() {
        return "PluginLoaderXmlRpcHandler";
    }

    public void initComponent() {
        myXmlRpcServer.addHandler(HANDLER_NAME, new PluginLoaderHandler());
    }

    public void disposeComponent() {
        myXmlRpcServer.removeHandler(HANDLER_NAME);
    }

    public class PluginLoaderHandler {
        /**
         * XML-RPC Method to check is plugin installed to find out perform next steps from broser side or not.
         * @return html content of response
         */
        public String installed() {
            return applyTemplate("Ok", "pluginLoader.status_reply.js");
        }

        /**
         * XML-RPC Method to load plugin. <strong>It should be protected in future to avoid not accepated plugins installations!!!</strong>
         * @param escapedPluginName name of the plugin.
         * @return html content of response
         */
        public String load(final String escapedPluginName) {
            LOG.debug("Loading of '" + escapedPluginName + "' started");
            //todo: is it a good way to decode a string?
            String decodedPluginName;
            try {
                decodedPluginName = URLDecoder.decode(escapedPluginName, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                decodedPluginName = escapedPluginName;
            }
            final String pluginName = decodedPluginName;
            final Application application = ApplicationManager.getApplication();

            final String[] result = new String[1];
            // Frame required for plugin installation.
            final Project[] openProjects =
                    ProjectManager.getInstance().getOpenProjects();
            if (openProjects.length == 0) {
                String message = "Cannot install when no IDEA projects opened. Please open a project and repeat.";
                LOG.debug(message);
                return applyTemplate(message, "pluginLoader.load_reply.js");
            }
            final Project project = openProjects[0];
            // Syncronous plugin installation.
            application.invokeAndWait(new Runnable() {
                public void run() {
                    // 6.X
                    ProgressManager.getInstance().runProcessWithProgressSynchronously(
                    // 5.X
//                    ApplicationManager.getApplication().runProcessWithProgressSynchronously(
                            new Runnable() {
                                public void run() {
                                    ProgressIndicator progressindicator = ProgressManager.getInstance().getProgressIndicator();
                                    PluginNode pluginNode = new PluginNode(PluginId.getId(pluginName));
                                    pluginNode.setName(pluginName);
                                    pluginNode.setSize("-1");
                                    progressindicator.setText(pluginNode.getName());
                                    try {
                                        LOG.debug("Plugin installation started");
                                        boolean installResult = PluginInstaller.prepareToInstall(pluginNode);
                                        if (installResult) {
                                            result[0] = "Plugin '" + pluginName + "' successfully downloaded!\\n" +
                                                    "Do not forget to restart IDEA to activate it.";
                                        } else {
                                            result[0] = "Plugin '" + pluginName + "' not found or not installed!";
                                        }
                                    } catch (IOException e) {
                                        result[0] = "Error downloading \"" + pluginName + "\" plugin from plugin site.";
                                    }
                                    LOG.debug(result[0]);
                                }
                            }, "Downloading plugin " + pluginName, true, project);

                }
            }, ModalityState.defaultModalityState());
            ApplicationManager.getApplication().invokeLater(new Runnable() {
                public void run() {
                    Messages.showInfoMessage(project, result[0], "Plugin installer");
                }
            });

            return applyTemplate(result[0], "pluginLoader.load_reply.js");
        }

        /**
         * Apply template. Tempate name is file name. Only one replacement possible for now.
         * @param replacement replacements
         * @param templateName template file name
         * @return
         */
        private String applyTemplate(String replacement, String templateName) {
            return getOutputTemplate(templateName).replaceAll(Pattern.quote("_replace_ment_"), Matcher.quoteReplacement(replacement));
        }

        /**
         * Get temaplte by name
         * @param templateName name
         * @return template
         */
        private String getOutputTemplate(String templateName) {
            String template = templates.get(templateName);
            if (StringUtil.isEmpty(template)) {
                template = readTemplate(templateName);
                templates.put(templateName, template);
            }
            return template;
        }

        /**
         * Fetch template by file name.
         * @param templateFileName file name (=template name for now)
         * @return template body
         */
        private String readTemplate(String templateFileName) {
            InputStream is = this.getClass().getResourceAsStream(templateFileName);
            String result = "";
            if (is != null) {
                BufferedReader reader = new BufferedReader(new InputStreamReader(is));
                String line;
                try {
                    while ((line = reader.readLine()) != null) {
                        result += line + "\n";
                    }
                } catch (IOException e) {
                    LOG.info("Error reading template " + templateFileName, e);
                } finally {
                    try {
                        reader.close();
                    } catch (IOException e) {
                    }
                }
            }
            return result;
        }
    }
}
